// Radix sort
#include <iostream>

// r, count array of size r (which is also the base, e.g., 2, 8, 10, 16)
// k, keys/digits (number of digits in the largest number to sort)
// n, size (number of elements in the array to sort)
// This algorithm requires k passes over the list of n numbers in base r
void radixsort(int A[], int k, int r, int n)
{
    int B[n];

    // This guarantees init to 0 for whole array, right?
    int count_arr[r]{0};

    int i, j, rtok;

    // Loop over place (0s, 10s, 100s, etc.)
    // Do a count sort within
    for(i = 0, rtok = 1;
        i < k; 
        i++, rtok *= r)
    {
        // Count the number of records in each bin on this pass
        for(j = 0; j < n; j++)
            count_arr[(A[j] / rtok) % r]++;

        // count[j] will be index in B
        // First, reduce count[0] because indexing starts at 0, not 1
        count_arr[0] = count_arr[0] - 1;

        // Prefix sum on count array
        for(j = 1; j < r; j++)
            count_arr[j] += count_arr[j - 1];

        // Put records into location, working from bottom
        // Since fill from bottom, j counts downwards
        for(j = n - 1; j >= 0; j--)
        {
            B[count_arr[(A[j] / rtok) % r]] = A[j];

            --count_arr[(A[j] / rtok) % r];
        }

        for (j = 0; j < n; j++)
            A[j] = B[j];  // Copy B back
    }
}

int main()
{
    int arr[] = {3, 2, 9, 4, 7};
    int n = 5;
    int r = 10;
    int k = 1;

    radixsort(arr, k, r, n);

    for(int c = 0; c < n; c++)
        std::cout << arr[c] << "\n";

    return 0;
}

